#! -*- coding: utf-8 -*-

import sys, random, time
import pygame
from pygame.locals import *
import escenario

COLORBASE = 0xDDEECC
CANTLADO = 25
LADOCELDA = 20

# ToDo:
# - Buscar el log de lo explicado a Lucio y hacer un post
# - Hacer que se de cuenta cuando termina, poniendo un lindo cartel y terminando
# - Hacer que las aristas puedan ser de menos de 20
# - Llevar a un escenario total de 640x480 (con escenario de 400x400)


class CentroComandos(object):
    def __init__(self, surface, colorbase, imagenes, botones, basex, altura):
        self.elegido = False
        self.surface = surface
        self.colorbase = colorbase
        self.sleepgo = .3

        # aca guardamos lo que tenemos para elegir
        # - x base
        # - y base
        # - ancho
        # - alto
        # - imagen
        self.imagenes = []

        # aca guardamos lo que pueden disparar
        # - x base
        # - y base
        # - ancho
        # - alto
        # - funcion
        self.botones = []

        # dibujamos las imagenes
        separador = 10
        altototal = sum(img.sfc.get_height() for img in imagenes) + separador*(len(imagenes)-1)
        masancha = max(img.sfc.get_width() for img in imagenes)
        posy = (altura-altototal) // 2
        for img in imagenes:
            surface.blit(img.sfc, (basex, posy))
            anch, alto = img.sfc.get_size()
            self.imagenes.append((basex, posy, anch, alto, img))
            posy += alto + separador
        pygame.display.update()
        
        # dibujamos los botones
        separador = 10
        basex = basex + masancha + 20
        altototal = sum(img.sfc.get_height() for img in botones) + separador*(len(botones)-1)
        posy = (altura-altototal) // 2
        for img in botones:
            surface.blit(img.sfc, (basex, posy))
            anch, alto = img.sfc.get_size()
            self.botones.append((basex, posy, anch, alto, getattr(self, img.nombre)))
            posy += alto + separador
        pygame.display.update()
        
    def limpiar(self):
        # redibujamos todo, para que limpie
        for (basex,basey,anch,alto, img) in self.imagenes:
            pygame.draw.rect(self.surface, self.colorbase, pygame.Rect(basex,basey,anch,alto), 3)
            surface.blit(img.sfc, (basex,basey))
        pygame.display.update()

    def clickIzq(self, pos):
        self.go = False
        # vemos si el click fue en algunas de las imagenes
        for (basex, basey, anch, alto, img) in self.imagenes:
            if basex <= pos[0] <= basex+anch and basey <= pos[1] <= basey+alto:
                # dibujamos la linea alrededor y devolvemos la imagen a usar
                self.limpiar()
                pygame.draw.rect(self.surface, 0xFFFFFF, pygame.Rect(basex,basey,anch,alto), 3)
                pygame.display.update()
                return img

       # vemos si el click fue en algunos de los botones 
        for (basex, basey, anch, alto, func) in self.botones:
            if basex <= pos[0] <= basex+anch and basey <= pos[1] <= basey+alto:
                func()

    def show_comida(self):
        terreno.showDispersion("Comida")
    def show_presencia(self):
        terreno.showDispersion("Hormiga")
    def show_hormiguero(self):
        terreno.showDispersion("Hormiguero")

    def simul_go(self):
        print "Go!"
        while True:
            time.sleep(self.sleepgo)
            self.simul_step()
            # manejamos los controles del automático
            evts = pygame.event.get([KEYDOWN])
            for e in evts:
                if e.type == KEYDOWN and e.key == K_PLUS:
                    self.sleepgo += .1
                    print "+ delay:", self.sleepgo
                if e.type == KEYDOWN and e.key == K_MINUS:
                    if self.sleepgo > .15:
                        self.sleepgo -= .1
                    print "- delay:", self.sleepgo
            # vemos si cortamos el loop
            if pygame.event.peek([QUIT, MOUSEBUTTONDOWN]):
                print "Saliendo del automático"
                break

    def simul_step(self):
        for elem in terreno.elementos.keys():
            if elem.es_bot:
                elem.step()
        

# Los elementos ven su alrededor como una lista de casilleros, en sentido
# horario desde 0:
#
#  7  0  1
#  6     2
#  5  4  3
#

# es el opuesto
OPUESTO = {0:4, 1:5, 2:6, 3:7, 4:0, 5:1, 6:2, 7:3}

# es cuanto hay que tocar x,y para ir hacia ese lado
DELTAS = {0:(0, -1), 1:(1, -1), 2:(1, 0), 3:(1, 1), 4:(0, 1), 5:(-1, 1), 6:(-1, 0), 7:(-1, -1)}

class Elemento(object):
    def __init__(self, x, y, propioyo):
        self.x = x
        self.y = y
        self.sfc = propioyo.sfc
        self.nombre = propioyo.nombre
        self.es_bot = False

class Hormiga(object):
    def __init__(self, x, y, propioyo):
        self.x = x
        self.y = y
        self.sfc = propioyo.sfc
        self.surfaces = propioyo.surfaces
        self.nombre = propioyo.nombre
        print "Hormiga agregándose en", x, y
        self.vengo_de = 0
        self.cargada = False
        self.es_bot = True

    def step(self):
#        print "Paso Hormiga %s (arrancando de %d %d)" % (self, self.x, self.y)
        prox = self._calc_prox()
#        print "Paso Hormiga %s (elegida direccion %d)" % (self, prox)
        if prox is None:
            return

        # nos movimos
        self.vengo_de = OPUESTO[prox]
        delta_x, delta_y = DELTAS[prox]
        new_x = self.x + delta_x
        new_y = self.y + delta_y

        # redibujamos
        pisamos = terreno.moverObjeto(self, new_x, new_y)
        self.x = new_x
        self.y = new_y
#        print "Paso Hormiga %s (coordenadas %d %d)" % (self, new_x, new_y)
#        print "Y pisamos...", pisamos
        
        # vemos si comimos algo
        for pis in pisamos:
            if pis.nombre == "Comida" and not self.cargada:
                self.cargada = True
                self.sfc = self.surfaces[1]
                terreno.sacarObjeto(pis)
                print "Boleteando", pis
            elif pis.nombre == "Hormiguero":
                self.cargada = False
                self.sfc = self.surfaces[0]
        return


    def _calc_prox(self):
        # Tomamos el alrededor del terreno; es distinto si buscamos
        # comida o el hormiguero
        if self.cargada:
            alrededor = terreno.getAlrededor(self.x, self.y, "Hormiguero")
        else:
            alrededor = terreno.getAlrededor(self.x, self.y, "Comida")
#        print "Alrededor:", alrededor

        # buscamos las posiciones de los máximos
        tope = max(alrededor)
        if tope is None:
            return
        postopes = [i for i,x in enumerate(alrededor) if x == tope]
#        print "Topes: (%d) %s" % (tope, postopes)

        # si hay uno solo, el camino es claro
        if len(postopes) == 1:
            return postopes[0]

        # si hay empate, preferimos inercia
        inerc = OPUESTO[self.vengo_de]
#        print "inercia", self.vengo_de, inerc
        if inerc in postopes:
            return inerc

        # y sino al azar, pero sin volver por nuestros pasos
#        print "Azar!:"
        if self.vengo_de in postopes:
            postopes.remove(self.vengo_de)
        return random.choice(postopes)

        
class Eventos(object):
    def __init__(self, limites_escenario):
        self.dragdrop = False
        self.flotando = None
        self.limminx,self.limminy,self.limmaxx,self.limmaxy = limites_escenario

    def _centrocomandos(self, event):
        if event.button == 1:
            elegido = cc.clickIzq(event.pos)
            if elegido is not None:
                self.dragdrop = True
                self.flotando = elegido
        elif event.button == 3:
            cc.limpiar()
            self.dragdrop = False
            self.flotando = None

    def _terreno(self, event):
        if event.button == 1:
            if self.dragdrop:
                terreno.tirarObjeto(self.flotando, event.pos)
        elif event.button == 3:
            terreno.vaciaBaldosa(event.pos)

    def raton_boton_up(self, event):
        terreno.hideDispersion()

    def raton_boton_down(self, event):
#        print event
        pos = event.pos
        if pos[0] > self.limmaxx-1:
            self._centrocomandos(event)
        elif self.limminx < pos[0] and (self.limminy <= pos[1] < self.limmaxy-1):
            self._terreno(event)


ladoescen = CANTLADO * LADOCELDA
alto = ladoescen + 10
ancho = ladoescen + 200
eventos = Eventos((5,5,ladoescen+5,ladoescen+5))

pygame.init()
window = pygame.display.set_mode((ancho, alto))
pygame.display.set_caption("Hormigas")
surface = pygame.display.get_surface()
surface.fill(COLORBASE)

terreno = escenario.Terreno(surface, arista=LADOCELDA, siz=(CANTLADO,CANTLADO), init=(5,5))
terreno.limpiarBaldosas()

class Receptor(object):
    def __init__(self, nombre, arch=None, sfc=None, clase=None):
        self.nombre = nombre
        if sfc is None and arch:
            if isinstance(arch, str):
                sfc = pygame.image.load(arch)
            else:
                self.surfaces = [pygame.image.load(x) for x in arch]
                sfc = self.surfaces[0]

        self.sfc = sfc
        self.clase = clase
        

# imagenes
font = pygame.font.Font(None, 17)
im_hormiga    = Receptor("Hormiga",         arch=["d_hormig_vacia.png","d_hormig_llena.png"], clase=Hormiga)
im_agua       = Receptor("Agua",            arch="d_agua.png", clase=Elemento)
im_comida     = Receptor("Comida",          arch="d_comida.png", clase=Elemento)
im_hormiguero = Receptor("Hormiguero",      arch="d_hormiguero.png", clase=Elemento)
bt_comida     = Receptor("show_comida",     sfc=font.render("Olor Comida", True, (0,0,127)))
bt_presencia  = Receptor("show_presencia",  sfc=font.render("Olor Presencia", True, (0,0,127)))
bt_hormiguero = Receptor("show_hormiguero", sfc=font.render("Olor Hormiguero", True, (0,0,127)))
bt_step       = Receptor("simul_step",      sfc=font.render("Paso", True, (0,0,127)))
bt_go         = Receptor("simul_go",        sfc=font.render("Go!", True, (0,0,127)))
cc = CentroComandos(surface, COLORBASE, [im_hormiga, im_comida, im_hormiguero, im_agua], 
                        [bt_presencia, bt_comida, bt_hormiguero, bt_step, bt_go], ladoescen + 20, alto)

funcevents = {
    QUIT: lambda x: sys.exit(),
    MOUSEBUTTONDOWN: eventos.raton_boton_down,
    MOUSEBUTTONUP:   eventos.raton_boton_up,
#    MOUSEMOTION:   eventos.raton_mov,
}


while True:
    event = pygame.event.wait()
    if event.type in funcevents:
        f = funcevents[event.type]
        f(event)
#    else:
#        print event.type
